import { useEditor } from "@/modules/editor";
import { css } from "@linaria/core";
import { defineComponent, ref } from "vue";
import { string } from "vue-types";
import { View } from "./View";

export const Text = defineComponent({
  props: {
    value: string().def("请输入文本"),
  },
  emits: ["update:value"],
  setup(props, { emit }) {
    const { store } = useEditor();
    const textRef = ref<HTMLTextAreaElement>();

    function textareaResize() {
      const textEl = textRef.value;
      if (textEl) {
        textEl.style.height = "0";
        textEl.style.height = textEl.scrollHeight + "px";
      }
    }

    return () => (
      <View class={textStyle}>
        {store.isEditMode ? (
          <textarea
            ref={textRef}
            value={props.value}
            rows={1}
            onInput={(e) => {
              emit("update:value", (e.target as HTMLInputElement).value);
              textareaResize();
            }}
          />
        ) : (
          <div class="whitespace-pre">{props.value}</div>
        )}
      </View>
    );
  },
});

const textStyle = css`
  text-align: left;
  textarea {
    resize: none;
  }
  textarea,
  div {
    display: block;
    width: 100%;
    height: 100%;
    padding: 4px;
    outline: none;
    border: none;
  }
`;
